from enum import Enum


class Trade(str, Enum):
    ORDER = "order"
    CANCEL = "cancel"
    CANCELBYCLOID = "cancelByCloid"
    SCHEDULECANCEL = "scheduleCancel"
    MODIFY = "modify"
    BATCHMODIFY = "batchModify"
    UPDATELEVERAGE = "updateLeverage"
    UPDATEISOLATEMARGIN = "updateIsolatedMargin"
    TWAPORDER = "twapOrder"
    TWAPCANCEL = "twapCancel"

    def __str__(self) -> str:
        return self.value

    def __repr__(self) -> str:
        return repr(self.value)



from enum import Enum


class Asset(str, Enum):
    GET_COIN_INFO = "/v5/asset/coin/query-info"
    GET_SUB_UID = "/v5/asset/transfer/query-sub-member-list"
    GET_SPOT_ASSET_INFO = "/v5/asset/transfer/query-asset-info"
    GET_ALL_COINS_BALANCE = "/v5/asset/transfer/query-account-coins-balance"
    GET_SINGLE_COIN_BALANCE = "/v5/asset/transfer/query-account-coin-balance"
    GET_WITHDRAWABLE_AMOUNT = "/v5/asset/withdraw/withdrawable-amount"
    GET_INTERNAL_TRANSFER_RECORDS = "/v5/asset/transfer/query-inter-transfer-list"
    GET_TRANSFERABLE_COIN = "/v5/asset/transfer/query-transfer-coin-list"
    CREATE_INTERNAL_TRANSFER = "/v5/asset/transfer/inter-transfer"
    CREATE_UNIVERSAL_TRANSFER = "/v5/asset/transfer/universal-transfer"
    GET_UNIVERSAL_TRANSFER_RECORDS = "/v5/asset/transfer/query-universal-transfer-list"
    SET_DEPOSIT_ACCOUNT = "/v5/asset/deposit/deposit-to-account"
    GET_DEPOSIT_RECORDS = "/v5/asset/deposit/query-record"
    GET_SUB_ACCOUNT_DEPOSIT_RECORDS = "/v5/asset/deposit/query-sub-member-record"
    GET_INTERNAL_DEPOSIT_RECORDS = "/v5/asset/deposit/query-internal-record"
    GET_MASTER_DEPOSIT_ADDRESS = "/v5/asset/deposit/query-address"
    GET_WITHDRAWAL_RECORDS = "/v5/asset/withdraw/query-record"
    WITHDRAW = "/v5/asset/withdraw/create"
    CANCEL_WITHDRAWAL = "/v5/asset/withdraw/cancel"

    def __str__(self) -> str:
        return self.value



from enum import Enum


class FutureMarket(str, Enum):
    GET_ALL_CONTRACTS = "/futures/{settle}/contracts"
    GET_A_SINGLE_CONTRACT = "/futures/{settle}/contracts/{contract}"
    ORDER_BOOK = "/futures/{settle}/order_book"
    GET_KLINE = "/futures/{settle}/candlesticks"
    LIST_TICKERS = "/futures/{settle}/tickers"
    FUNDING_RATE_HISTORY = "/futures/{settle}/funding_rate"

    def __str__(self) -> str:
        return self.value


class DeliveryMarket(str, Enum):
    GET_ALL_CONTRACTS = "/delivery/{settle}/contracts"
    ORDER_BOOK = "/delivery/{settle}/order_book"
    GET_KLINE = "/delivery/{settle}/candlesticks"
    LIST_TICKERS = "/delivery/{settle}/tickers"


class SpotMarket(str, Enum):
    GET_ALL_CURRENCY_PAIRS = "/spot/currency_pairs"
    ORDER_BOOK = "/spot/order_book"
    GET_KLINE = "/spot/candlesticks"
    LIST_TICKERS = "/spot/tickers"

    def __str__(self) -> str:
        return self.value



import hmac
import json
import logging
import httpx
import hashlib
from dataclasses import dataclass, field
from .endpoints.account import FundingAccount, FuturesAccount
from .endpoints.market import FuturesMarket, SpotMarket
from .endpoints.trade import FuturesTrade, SpotTrade
from ..product_table.manager import ProductTableManager
from ...utils.errors import FailedRequestError
from ...utils.helpers import generate_timestamp
from ...utils.common import Common


def sign_message(timestamp, memo, body, secret_key):
    message = f"{timestamp}#{memo}#{body}"
    return hmac.new(secret_key.encode(), message.encode(), hashlib.sha256).hexdigest()


def get_header(api_key, sign, timestamp, memo):
    return {
        "Content-Type": "application/json",
        "X-BM-KEY": api_key,
        "X-BM-SIGN": sign,
        "X-BM-TIMESTAMP": str(timestamp),
        "X-BM-MEMO": memo,
    }


def get_header_no_sign():
    return {"Content-Type": "application/json"}


@dataclass
class HTTPManager:
    api_key: str = field(default=None)
    api_secret: str = field(default=None)
    memo: str = field(default=None)
    timeout: int = field(default=10)
    max_retries: int = field(default=3)
    retry_delay: int = field(default=3)
    logger: logging.Logger = field(default=None)
    session: httpx.AsyncClient = field(init=False)
    ptm: ProductTableManager = field(init=False)
    preload_product_table: bool = field(default=True)

    api_map = {
        "https://api-cloud.bitmart.com": {
            SpotTrade,
            SpotMarket,
            FundingAccount,
        },  # v1 API
        "https://api-cloud-v2.bitmart.com": {
            FuturesTrade,
            FuturesMarket,
            FuturesAccount,
        },  # v2 API
    }

    async def async_init(self):
        self.session = httpx.AsyncClient(timeout=self.timeout)
        self._logger = self.logger or logging.getLogger(__name__)
        if self.preload_product_table:
            self.ptm = await ProductTableManager.get_instance(Common.BITMART)
        return self

    def _get_base_url(self, path):
        for base_url, enums in self.api_map.items():
            if type(path) in enums:
                return base_url
        raise ValueError(f"Unknown API path: {path} (type={type(path)})")

    async def _request(
        self,
        method: str,
        path: str,
        query: dict = None,
        signed: bool = True,
    ):
        if not self.session:
            await self.async_init()

        if query is None:
            query = {}

        base_url = self._get_base_url(path)
        url = base_url + path.value

        if method.upper() == "GET" and query:
            params_str = "&".join(f"{k}={v}" for k, v in sorted(query.items()) if v)
            url = f"{url}?{params_str}"

        timestamp = generate_timestamp()

        if signed:
            if not (self.api_key and self.api_secret and self.memo):
                raise ValueError("Signed request requires API Key and Secret and Memo.")
            sign = sign_message(timestamp, self.memo, json.dumps(query), self.api_secret)
            headers = get_header(self.api_key, sign, timestamp, self.memo)
        else:
            headers = get_header_no_sign()

        try:
            if method.upper() == "GET":
                response = await self.session.get(url, headers=headers)
            elif method.upper() == "POST":
                response = await self.session.post(
                    url,
                    json=query if query else {},
                    headers=headers,
                )
            else:
                raise ValueError(f"Unsupported HTTP method: {method}")

            try:
                data = response.json()
            except Exception:
                data = {}

            if data.get("code", 0) != 1000:
                code = data.get("code", "Unknown")
                error_msg = data.get("message", "Unknown error")
                raise FailedRequestError(
                    request=f"{method.upper()} {url} | Body: {query}",
                    message=f"BitMart API Error: [{code}] {error_msg}",
                    status_code=response.status_code,
                    time=timestamp,
                    resp_headers=response.headers,
                )

            # If http status is not 2xx (like 403, 404)
            if not response.status_code // 100 == 2:
                raise FailedRequestError(
                    request=f"{method.upper()} {url} | Body: {query}",
                    message=f"HTTP Error {response.status_code}: {response.text}",
                    status_code=response.status_code,
                    time=timestamp,
                    resp_headers=response.headers,
                )

            return data

        except httpx.HTTPError as e:
            raise FailedRequestError(
                request=f"{method.upper()} {url} | Body: {query}",
                message=f"Request failed: {str(e)}",
                status_code=response.status_code if response else "Unknown",
                time=timestamp,
                resp_headers=response.headers if response else None,
            )



# Import exchange client classes and create callable functions
from .bybit.client import Client as BybitClient
from .binance.client import Client as BinanceClient
from .okx.client import Client as OKXClient
from .bitmart.client import Client as BitmartClient
from .gateio.client import Client as GateioClient
from .hyperliquid.client import Client as HyperliquidClient


async def bybit(**kwargs):
    """Create and initialize a Bybit client instance."""
    client = BybitClient(**kwargs)
    await client.async_init()
    return client

async def binance(**kwargs):
    """Create and initialize a Binance client instance."""
    client = BinanceClient(**kwargs)
    await client.async_init()
    return client

async def okx(**kwargs):
    """Create and initialize an OKX client instance."""
    client = OKXClient(**kwargs)
    await client.async_init()
    return client

async def bitmart(**kwargs):
    """Create and initialize a BitMart client instance."""
    client = BitmartClient(**kwargs)
    await client.async_init()
    return client

async def gateio(**kwargs):
    """Create and initialize a Gate.io client instance."""
    client = GateioClient(**kwargs)
    await client.async_init()
    return client

async def hyperliquid(**kwargs):
    """Create and initialize a Hyperliquid client instance."""
    client = HyperliquidClient(**kwargs)
    await client.async_init()
    return client

__all__ = [
    "bybit", "binance", "okx", "bitmart", "gateio", "hyperliquid",
]



from ._http_manager import HTTPManager
from .endpoints.trade import DeliveryTrade, FutureTrade, SpotTrade
from ..utils.common import Common


class TradeHTTP(HTTPManager):
    def get_futures_all_positions(
        self,
        ccy: str = "usdt",  # or "btc"
        holding: bool = False,
        limit: int = None,
        offset: int = None,
    ):
        """
        :param ccy: str
        :param holding: bool
        :param limit: int
        :param offset: int
        """
        path_params = {
            "settle": ccy,
        }

        payload = {
            "holding": holding,
        }
        if limit:
            payload["limit"] = limit
        if offset:
            payload["offset"] = offset

        res = self._request(
            method="GET",
            path=FutureTrade.GET_ALL_POSITIONS,
            path_params=path_params,
            query=payload,
        )
        return res

    def get_contract_single_positions(
        self,
        product_symbol: str,
        ccy: str = "usdt",  # or "btc" (futures)
        path: str = "futures",
    ):
        """
        :param product_symbol: str
        :param ccy: str
        """
        path_params = {
            "settle": ccy,
            "contract": self.ptm.get_exchange_symbol(Common.GATEIO, product_symbol),
        }

        if path == "futures":
            path_ = FutureTrade.GET_SINGLE_POSITION
        elif path == "delivery":
            path_ = DeliveryTrade.GET_SINGLE_POSITION
        else:
            raise ValueError(f"Unsupported path: {path}")

        res = self._request(
            method="GET",
            path=path_,
            path_params=path_params,
        )
        return res

    def update_futures_positions_leverage(
        self,
        product_symbol: str,
        leverage: str,
        ccy: str = "usdt",  # or "btc"
        cross_leverage_limit: str = None,
    ):
        """
        :param product_symbol: str
        :param leverage: str
        :param ccy: str
        :param cross_leverage_limit: str (Cross margin leverage(valid only when leverage is 0))
        """
        path_params = {
            "settle": ccy,
            "contract": self.ptm.get_exchange_symbol(Common.GATEIO, product_symbol),
        }

        payload = {
            "leverage": leverage,
        }

        if cross_leverage_limit:
            payload["cross_leverage_limit"] = cross_leverage_limit

        res = self._request(
            method="POST",
            path=FutureTrade.UPDATE_POSITION_LEVERAGE,
            path_params=path_params,
            query=payload,
        )
        return res

    def future_dual_mode_switch(
        self,
        dual_mode: bool,
        ccy: str = "usdt",  # or "btc"
    ):
        """
        :param dual_mode: bool
        :param ccy: str
        """
        path_params = {
            "settle": ccy,
        }

        payload = {
            "dual_mode": dual_mode,
        }

        res = self._request(
            method="POST",
            path=FutureTrade.DUAL_MODE_SWITCH,
            path_params=path_params,
            query=payload,
        )
        return res

    def place_contract_order(
        self,
        product_symbol: str,
        size: int,
        ccy: str = "usdt",  # or "btc"
        path: str = "futures",
        iceberg: int = None,
        price: str = None,
        close: bool = None,
        reduce_only: bool = None,
        tif: str = None,
        text: str = None,
        auto_size: str = None,
        stp_act: str = None,
    ):
        """
        :param product_symbol: str
        :param size: int
        :param ccy: str
        :param path: str
        :param iceberg: int
        :param price: str
        :param close: bool
        :param reduce_only: bool
        :param tif: str
        :param text: str
        :param auto_size: str
        :param stp_act: str
        """

        path_params = {
            "settle": ccy,
        }

        body = {
            "contract": self.ptm.get_exchange_symbol(Common.GATEIO, product_symbol),
            "size": size,
        }

        if iceberg is not None:
            body["iceberg"] = iceberg
        if price is not None:
            body["price"] = price
        if close is not None:
            body["close"] = close
        if reduce_only is not None:
            body["reduce_only"] = reduce_only
        if tif is not None:
            body["tif"] = tif
        if text is not None:
            body["text"] = text
        if auto_size is not None:
            body["auto_size"] = auto_size
        if stp_act is not None:
            body["stp_act"] = stp_act

        if path == "futures":
            path_ = FutureTrade.FUTURES_ORDER
        elif path == "delivery":
            path_ = DeliveryTrade.FUTURES_ORDER
        else:
            raise ValueError(f"Unsupported path: {path}")

        res = self._request(
            method="POST",
            path=path_,
            path_params=path_params,
            body=body,
        )
        return res

    def place_contract_market_order(
        self,
        product_symbol: str,
        size: int,
        ccy: str = "usdt",
        path: str = "futures",
    ):
        return self.place_contract_order(
            product_symbol=product_symbol,
            size=size,
            price="0",
            tif="ioc",
            ccy=ccy,
            path=path,
        )

    def place_contract_market_buy_order(
        self,
        product_symbol: str,
        size: int,
        ccy: str = "usdt",
        path: str = "futures",
    ):
        return self.place_contract_market_order(
            product_symbol=product_symbol,
            size=abs(size),
            ccy=ccy,
            path=path,
        )

    def place_contract_market_sell_order(
        self,
        product_symbol: str,
        size: int,
        ccy: str = "usdt",
        path: str = "futures",
    ):
        return self.place_contract_market_order(
            product_symbol=product_symbol,
            size=-abs(size),  # negative size for sell
            ccy=ccy,
            path=path,
        )

    def place_contract_limit_order(
        self,
        product_symbol: str,
        size: int,
        price: str,
        ccy: str = "usdt",
        path: str = "futures",
    ):
        return self.place_contract_order(
            product_symbol=product_symbol,
            size=size,
            price=price,
            tif="gtc",
            ccy=ccy,
            path=path,
        )

    def place_contract_limit_buy_order(
        self,
        product_symbol: str,
        size: int,
        price: str,
        ccy: str = "usdt",
        path: str = "futures",
    ):
        return self.place_contract_limit_order(
            product_symbol=product_symbol,
            size=abs(size),
            price=price,
            ccy=ccy,
            path=path,
        )

    def place_contract_limit_sell_order(
        self,
        product_symbol: str,
        size: int,
        price: str,
        ccy: str = "usdt",
        path: str = "futures",
    ):
        return self.place_contract_limit_order(
            product_symbol=product_symbol,
            size=-abs(size),
            price=price,
            ccy=ccy,
            path=path,
        )

    def place_contract_post_only_limit_order(
        self,
        product_symbol: str,
        size: int,
        price: str,
        ccy: str = "usdt",
        path: str = "futures",
    ):
        return self.place_contract_order(
            product_symbol=product_symbol,
            size=size,
            price=price,
            tif="poc",
            ccy=ccy,
            path=path,
        )

    def place_contract_post_only_limit_buy_order(
        self,
        product_symbol: str,
        size: int,
        price: str,
        ccy: str = "usdt",
        path: str = "futures",
    ):
        return self.place_contract_post_only_limit_order(
            product_symbol=product_symbol,
            size=abs(size),
            price=price,
            ccy=ccy,
            path=path,
        )

    def place_contract_post_only_limit_sell_order(
        self,
        product_symbol: str,
        size: int,
        price: str,
        ccy: str = "usdt",
        path: str = "futures",
    ):
        return self.place_contract_post_only_limit_order(
            product_symbol=product_symbol,
            size=-abs(size),
            price=price,
            ccy=ccy,
            path=path,
        )

    def place_futures_batch_order(
        self,
        orders: list[dict],
        ccy: str = "usdt",  # or "btc"
    ):
        """
        :param orders: list[dict]
        :param ccy: str
        """
        if not isinstance(orders, list) or not all(isinstance(o, dict) for o in orders):
            raise TypeError("Orders must be a list of dictionaries.")

        if len(orders) > 10:
            raise ValueError("The number of orders cannot exceed 10.")

        for order in orders:
            if "contract" not in order and "product_symbol" in order:
                order["contract"] = self.ptm.get_exchange_symbol(Common.GATEIO, order["product_symbol"])
                del order["product_symbol"]

        path_params = {
            "settle": ccy,
        }

        res = self._request(
            method="POST",
            path=FutureTrade.BATCH_FUTURES_ORDERS,
            path_params=path_params,
            body=orders,
        )
        return res

    def get_contract_order_list(
        self,
        status: str,
        ccy: str = "usdt",  # or "btc"
        path: str = "futures",
        product_symbol: str = None,
        limit: int = None,
        offset: int = None,
        last_id: str = None,
        count_total: int = None,
    ):
        """
        :param status: str
        :param ccy: str
        :param path: str
        :param product_symbol: str
        :param limit: int
        :param offset: int
        :param last_id: str
        :param count_total: int
        """
        path_params = {
            "settle": ccy,
        }

        payload = {
            "status": status,
        }
        if product_symbol is not None:
            payload["contract"] = self.ptm.get_exchange_symbol(Common.GATEIO, product_symbol)
        if limit is not None:
            payload["limit"] = limit
        if offset is not None:
            payload["offset"] = offset
        if last_id is not None:
            payload["last_id"] = last_id

        if path == "futures":
            path_ = FutureTrade.FUTURES_ORDER
        elif path == "delivery":
            path_ = DeliveryTrade.FUTURES_ORDER

            if count_total is not None:
                payload["count_total"] = count_total
        else:
            raise ValueError(f"Unsupported path: {path}")

        res = self._request(
            method="GET",
            path=path_,
            path_params=path_params,
            query=payload,
        )
        return res

    def cancel_contract_all_order_matched(
        self,
        product_symbol: str,
        ccy: str = "usdt",  # or "btc"
        path: str = "futures",
        side: str = None,
    ):
        """
        :param product_symbol: str
        :param ccy: str
        :param path: str
        :param side: str
        """
        path_params = {
            "settle": ccy,
        }

        payload = {
            "contract": self.ptm.get_exchange_symbol(Common.GATEIO, product_symbol),
        }
        if side is not None:
            payload["side"] = side

        if path == "futures":
            path_ = FutureTrade.FUTURES_ORDER
        elif path == "delivery":
            path_ = DeliveryTrade.FUTURES_ORDER
        else:
            raise ValueError(f"Unsupported path: {path}")

        res = self._request(
            method="DELETE",
            path=path_,
            path_params=path_params,
            query=payload,
        )
        return res

    def get_contract_single_order(
        self,
        order_id: str,
        ccy: str = "usdt",  # or "btc"
        path: str = "futures",
    ):
        """
        :param order_id: str
        :param ccy: str
        :param path: str
        """
        path_params = {
            "settle": ccy,
            "order_id": order_id,
        }

        if path == "futures":
            path_ = FutureTrade.SINGLE_ORDER
        elif path == "delivery":
            path_ = DeliveryTrade.SINGLE_ORDER
        else:
            raise ValueError(f"Unsupported path: {path}")

        res = self._request(
            method="GET",
            path=path_,
            path_params=path_params,
        )
        return res

    def cancel_contract_single_order(
        self,
        order_id: str,
        ccy: str = "usdt",  # or "btc"
        path: str = "futures",
    ):
        """
        :param order_id: str
        :param ccy: str
        :param path: str
        """
        path_params = {
            "settle": ccy,
            "order_id": order_id,
        }

        if path == "futures":
            path_ = FutureTrade.SINGLE_ORDER
        elif path == "delivery":
            path_ = DeliveryTrade.SINGLE_ORDER
        else:
            raise ValueError(f"Unsupported path: {path}")

        res = self._request(
            method="DELETE",
            path=path_,
            path_params=path_params,
        )
        return res

    def amend_futures_single_order(
        self,
        order_id: str,
        ccy: str = "usdt",  # or "btc"
        size: int = None,
        price: str = None,
        amend_text: str = None,
        biz_info: str = None,
        bbo: str = None,
    ):
        """
        :param order_id: str
        :param ccy: str
        :param size: int
        :param price: str
        :param amend_text: str
        :param biz_info: str
        :param bbo: str
        """
        path_params = {
            "settle": ccy,
            "order_id": order_id,
        }

        body = {}
        if size is not None:
            body["size"] = size
        if price is not None:
            body["price"] = price
        if amend_text is not None:
            body["amend_text"] = amend_text
        if biz_info is not None:
            body["biz_info"] = biz_info
        if bbo is not None:
            body["bbo"] = bbo

        res = self._request(
            method="PUT",
            path=FutureTrade.SINGLE_ORDER,
            path_params=path_params,
            body=body,
        )
        return res

    def get_trading_history(
        self,
        product_symbol: str,
        ccy: str = "usdt",  # or "btc"
        path: str = "futures",
        order: str = None,
        limit: int = None,
        offset: int = None,
        late_id: str = None,
        count_total: int = None,
    ):
        """
        :param product_symbol: str
        :param ccy: str
        :param path: str
        :param order: str
        :param limit: int
        :param offset: int
        :param late_id: str
        :param count_total: int
        """
        path_params = {
            "settle": ccy,
        }

        payload = {
            "contract": self.ptm.get_exchange_symbol(Common.GATEIO, product_symbol),
        }

        if order is not None:
            payload["order"] = order
        if limit is not None:
            payload["limit"] = limit
        if offset is not None:
            payload["offset"] = offset
        if late_id is not None:
            payload["late_id"] = late_id

        if path == "futures":
            path_ = FutureTrade.LIST_PERSONAL_TRADING_HISTORY
        elif path == "delivery":
            path_ = DeliveryTrade.LIST_PERSONAL_TRADING_HISTORY

            if count_total is not None:
                payload["count_total"] = count_total
        else:
            raise ValueError(f"Unsupported path: {path}")

        res = self._request(
            method="GET",
            path=path_,
            path_params=path_params,
            query=payload,
        )
        return res

    def get_futures_position_close_history(
        self,
        product_symbol: str,
        ccy: str = "usdt",  # or "btc"
        limit: int = None,
        offset: int = None,
        from_timestamp: int = None,
        to_timestamp: int = None,
        side: str = None,
        pnl: str = None,
    ):
        """
        :param product_symbol: str
        :param ccy: str
        :param limit: int
        :param offset: int
        :param from_timestamp: int
        :param to_timestamp: int
        :param side: str
        :param pnl: str
        """
        path_params = {
            "settle": ccy,
        }

        payload = {
            "contract": self.ptm.get_exchange_symbol(Common.GATEIO, product_symbol),
        }

        if limit is not None:
            payload["limit"] = limit
        if offset is not None:
            payload["offset"] = offset
        if from_timestamp is not None:
            payload["from"] = from_timestamp
        if to_timestamp is not None:
            payload["to"] = to_timestamp
        if side is not None:
            payload["side"] = side
        if pnl is not None:
            payload["pnl"] = pnl

        res = self._request(
            method="GET",
            path=FutureTrade.LIST_POSITION_CLOSE_HISTORY,
            path_params=path_params,
            query=payload,
        )
        return res

    def get_futures_auto_deleveraging_history(
        self,
        product_symbol: str,
        ccy: str = "usdt",  # or "btc"
        limit: int = None,
        at_timestamp: int = None,
    ):
        """
        :param product_symbol: str
        :param ccy: str
        :param limit: int
        :param at_timestamp: int
        """
        path_params = {
            "settle": ccy,
        }

        payload = {
            "contract": self.ptm.get_exchange_symbol(Common.GATEIO, product_symbol),
        }

        if limit is not None:
            payload["limit"] = limit
        if at_timestamp is not None:
            payload["at"] = at_timestamp

        res = self._request(
            method="GET",
            path=FutureTrade.LIST_AUTODELEVERAGING_HISTORY,
            path_params=path_params,
            query=payload,
        )
        return res

    def get_delivery_all_positions(
        self,
        ccy: str = "usdt",
    ):
        """
        :param ccy: str
        """
        path_params = {
            "settle": ccy,
        }

        res = self._request(
            method="GET",
            path=DeliveryTrade.GET_ALL_POSITIONS,
            path_params=path_params,
        )
        return res

    def update_delivery_positions_leverage(
        self,
        product_symbol: str,
        leverage: str,
        ccy: str = "usdt",
    ):
        """
        :param product_symbol: str
        :param leverage: str
        :param ccy: str
        """
        path_params = {
            "settle": ccy,
            "contract": self.ptm.get_exchange_symbol(Common.GATEIO, product_symbol),
        }

        payload = {
            "leverage": leverage,
        }

        res = self._request(
            method="POST",
            path=DeliveryTrade.UPDATE_POSITION_LEVERAGE,
            path_params=path_params,
            query=payload,
        )
        return res

    def get_delivery_position_close_history(
        self,
        product_symbol: str,
        ccy: str = "usdt",  # or "btc"
        limit: int = None,
    ):
        """
        :param product_symbol: str
        :param ccy: str
        :param limit: int
        """
        path_params = {
            "settle": ccy,
        }

        payload = {
            "contract": self.ptm.get_exchange_symbol(Common.GATEIO, product_symbol),
        }

        if limit is not None:
            payload["limit"] = limit

        res = self._request(
            method="GET",
            path=DeliveryTrade.LIST_POSITION_CLOSE_HISTORY,
            path_params=path_params,
            query=payload,
        )
        return res

    def place_spot_order(
        self,
        product_symbol: str,
        side: str,
        amount: str,
        text: str = None,
        order_type: str = None,  # limit or market
        account: str = None,  # spot, margin, unified
        price: str = None,
        time_in_force: str = None,  # gtc, ioc, poc, fok
        iceberg: str = None,
        auto_borrow: bool = False,
        auto_repay: bool = False,
        stp_act: str = None,
        action_mode: str = None,  # ACK, RESULT, FULL
    ):
        """
        :param product_symbol: str
        :param side: str
        :param amount: str
        :param text: str
        :param order_type: str
        :param account: str
        :param price: str
        :param time_in_force: str
        :param iceberg: str
        :param auto_borrow: bool
        :param auto_repay: bool
        :param stp_act: str
        :param action_mode: str
        """

        body = {
            "currency_pair": self.ptm.get_exchange_symbol(Common.GATEIO, product_symbol),
            "side": side,
            "amount": amount,
        }

        if text is not None:
            body["text"] = text
        if order_type is not None:
            body["type"] = order_type
        if account is not None:
            body["account"] = account
        if price is not None:
            body["price"] = price
        if time_in_force is not None:
            body["time_in_force"] = time_in_force
        if iceberg is not None:
            body["iceberg"] = iceberg
        if auto_borrow:
            body["auto_borrow"] = True
        if auto_repay:
            body["auto_repay"] = True
        if stp_act is not None:
            body["stp_act"] = stp_act
        if action_mode is not None:
            body["action_mode"] = action_mode

        res = self._request(
            method="POST",
            path=SpotTrade.SPOT_ORDER,
            body=body,
        )
        return res

    def place_market_order(
        self,
        product_symbol: str,
        side: str,
        amount: str,
    ):
        return self.place_spot_order(
            product_symbol=product_symbol,
            side=side,
            order_type="market",
            time_in_force="ioc",
            amount=amount,
        )

    def place_market_buy_order(
        self,
        product_symbol: str,
        amount: str,
    ):
        return self.place_market_order(
            product_symbol=product_symbol,
            side="buy",
            amount=amount,
        )

    def place_market_sell_order(
        self,
        product_symbol: str,
        amount: str,
    ):
        return self.place_market_order(
            product_symbol=product_symbol,
            side="sell",
            amount=amount,
        )

    def place_limit_order(
        self,
        product_symbol: str,
        side: str,
        amount: str,
        price: str,
    ):
        return self.place_spot_order(
            product_symbol=product_symbol,
            side=side,
            order_type="limit",
            amount=amount,
            price=price,
        )

    def place_limit_buy_order(
        self,
        product_symbol: str,
        amount: str,
        price: str,
    ):
        return self.place_limit_order(
            product_symbol=product_symbol,
            side="buy",
            amount=amount,
            price=price,
        )

    def place_limit_sell_order(
        self,
        product_symbol: str,
        amount: str,
        price: str,
    ):
        return self.place_limit_order(
            product_symbol=product_symbol,
            side="sell",
            amount=amount,
            price=price,
        )

    def place_post_only_limit_order(
        self,
        product_symbol: str,
        side: str,
        amount: str,
        price: str,
    ):
        return self.place_spot_order(
            product_symbol=product_symbol,
            side=side,
            order_type="limit",
            time_in_force="poc",
            amount=amount,
            price=price,
        )

    def place_post_only_limit_buy_order(
        self,
        product_symbol: str,
        amount: str,
        price: str,
    ):
        return self.place_post_only_limit_order(
            product_symbol=product_symbol,
            side="buy",
            amount=amount,
            price=price,
        )

    def place_post_only_limit_sell_order(
        self,
        product_symbol: str,
        amount: str,
        price: str,
    ):
        return self.place_post_only_limit_order(
            product_symbol=product_symbol,
            side="sell",
            amount=amount,
            price=price,
        )

    def get_spot_open_orders(
        self,
        page: str = None,
        limit: str = None,
        account: str = None,
    ):
        """
        :param page: str
        :param limit: str
        :param account: str
        """

        payload = {}

        if page is not None:
            payload["page"] = page
        if limit is not None:
            payload["limit"] = limit
        if account is not None:
            payload["account"] = account

        res = self._request(
            method="GET",
            path=SpotTrade.GET_OPEN_ORDER,
            query=payload,
        )
        return res

    def get_spot_order_list(
        self,
        product_symbol: str,
        status: str,
        page: str = None,
        limit: str = None,
        account: str = None,
        from_timestamp: str = None,
        to_timestamp: str = None,
        side: str = None,
    ):
        """
        :param product_symbol: str
        :param status: str
        :param page: str
        :param limit: str
        :param account: str
        :param from_timestamp: str
        :param to_timestamp: str
        :param side: str
        """

        payload = {
            "currency_pair": self.ptm.get_exchange_symbol(Common.GATEIO, product_symbol),
            "status": status,
        }

        if page is not None:
            payload["page"] = page
        if limit is not None:
            payload["limit"] = limit
        if account is not None:
            payload["account"] = account
        if from_timestamp is not None:
            payload["from"] = from_timestamp
        if to_timestamp is not None:
            payload["to"] = to_timestamp
        if side is not None:
            payload["side"] = side

        res = self._request(
            method="GET",
            path=SpotTrade.SPOT_ORDER,
            query=payload,
        )
        return res

    def cancel_spot_order(
        self,
        product_symbol: str = None,
        side: str = None,
        account: str = None,
        action_mode: str = None,
    ):
        """
        :param product_symbol: str
        :param side: str
        :param account: str
        :param action_mode: str
        """
        payload = {}
        if product_symbol is not None:
            payload["currency_pair"] = self.ptm.get_exchange_symbol(Common.GATEIO, product_symbol)
        if side is not None:
            payload["side"] = side
        if account is not None:
            payload["account"] = account
        if action_mode is not None:
            payload["action_mode"] = action_mode

        res = self._request(
            method="DELETE",
            path=SpotTrade.SPOT_ORDER,
            query=payload,
        )
        return res

    def get_spot_single_order(
        self,
        order_id: str,
        product_symbol: str,
        account: str = None,
    ):
        """
        :param order_id: str
        :param product_symbol: str
        :param account: str
        """
        path_params = {
            "order_id": order_id,
        }

        payload = {
            "currency_pair": self.ptm.get_exchange_symbol(Common.GATEIO, product_symbol),
        }
        if account is not None:
            payload["account"] = account

        res = self._request(
            method="GET",
            path=SpotTrade.SINGLE_ORDER,
            path_params=path_params,
            query=payload,
        )
        return res

    def cancel_spot_single_order(
        self,
        order_id: str,
        product_symbol: str,
        account: str = None,
        action_mode: str = None,
    ):
        """
        :param order_id: str
        :param product_symbol: str
        :param account: str
        :param action_mode: str
        """
        path_params = {
            "order_id": order_id,
        }

        payload = {
            "currency_pair": self.ptm.get_exchange_symbol(Common.GATEIO, product_symbol),
        }
        if account is not None:
            payload["account"] = account
        if action_mode is not None:
            payload["action_mode"] = action_mode

        res = self._request(
            method="DELETE",
            path=SpotTrade.SINGLE_ORDER,
            path_params=path_params,
            query=payload,
        )
        return res

    def amend_spot_single_order(
        self,
        order_id: str,
        product_symbol: int = None,
        account: str = None,
        amount: str = None,
        price: str = None,
        amend_text: str = None,
        action_mode: str = None,
    ):
        """
        :param order_id: str
        :param product_symbol: int
        :param account: str
        :param amount: str
        :param price: str
        :param amend_text: str
        :param action_mode: str
        """
        path_params = {
            "order_id": order_id,
        }

        body = {}
        if product_symbol is not None:
            body["currency_pair"] = self.ptm.get_exchange_symbol(Common.GATEIO, product_symbol)
        if account is not None:
            body["account"] = account
        if amount is not None:
            body["amount"] = amount
        if price is not None:
            body["price"] = price
        if amend_text is not None:
            body["amend_text"] = amend_text
        if action_mode is not None:
            body["action_mode"] = action_mode

        res = self._request(
            method="PATCH",
            path=SpotTrade.SINGLE_ORDER,
            path_params=path_params,
            body=body,
        )
        return res

    def get_spot_trading_history(
        self,
        product_symbol: str = None,
        limit: int = None,
        page: int = None,
        order_id: str = None,
        account: str = None,
        from_timestamp: int = None,
        to_timestamp: int = None,
    ):
        """
        :param product_symbol: str
        :param limit: int
        :param page: int
        :param order_id: str
        :param account: str
        :param from_timestamp: int
        :param to_timestamp: int
        """
        payload = {}
        if product_symbol is not None:
            payload["currency_pair"] = self.ptm.get_exchange_symbol(Common.GATEIO, product_symbol)
        if limit is not None:
            payload["limit"] = limit
        if page is not None:
            payload["page"] = page
        if order_id is not None:
            payload["order_id"] = order_id
        if account is not None:
            payload["account"] = account
        if from_timestamp is not None:
            payload["from"] = from_timestamp
        if to_timestamp is not None:
            payload["to"] = to_timestamp

        res = self._request(
            method="GET",
            path=SpotTrade.LIST_PERSONAL_TRADING_HISTORY,
            query=payload,
        )
        return res



from ._http_manager import HTTPManager
from .endpoints.account import Account
from ...utils.common import Common


class AccountHTTP(HTTPManager):
    async def get_wallet_balance(self):
        """
        async default UNIFIED account and ensure the account is upgraded to unified account before trading
        """
        payload = {
            "accountType": "UNIFIED",
        }

        res = await self._request(
            method="GET",
            path=Account.GET_WALLET_BALANCE,
            query=payload,
        )
        return res

    async def get_transferable_amount(
        self,
        coins: list,
    ):
        """
        :param coins: list
        """
        payload = {}
        if coins is not None:
            coinName = ",".join(coins)
            payload = {
                "coinName": coinName,
            }

        res = await self._request(
            method="GET",
            path=Account.GET_TRANSFERABLE_AMOUNT,
            query=payload,
        )
        return res

    async def upgrade_to_unified_trading_account(self):
        res = await self._request(
            method="POST",
            path=Account.UPGRADE_TO_UNIFIED_ACCOUNT,
            query=None,
        )
        return res

    async def get_borrow_history(
        self,
        coin: str = None,
        startTime: int = None,
        limit: int = 20,
    ):
        """
        :param coin: str
        :param startTime: int
        :param limit: int
        """
        payload = {
            "limit": limit,
        }
        if coin is not None:
            payload["currency"] = coin
        if startTime is not None:
            payload["startTime"] = startTime

        res = await self._request(
            method="GET",
            path=Account.GET_BORROW_HISTORY,
            query=payload,
        )
        return res

    async def repay_liability(
        self,
        coin: str = None,
    ):
        """
        :param coin: str
        """
        payload = {}
        if coin is not None:
            payload = {
                "coin": coin,
            }

        res = await self._request(
            method="POST",
            path=Account.REPAY_LIABILITY,
            query=payload,
        )
        return res

    async def get_collateral_info(
        self,
        coin: str = None,
    ):
        """
        :param coin: str
        """
        payload = {}
        if coin is not None:
            payload = {
                "coin": coin,
            }

        res = await self._request(
            method="GET",
            path=Account.GET_COLLATERAL_INFO,
            query=payload,
        )
        return res

    async def set_collateral_coin(
        self,
        coin: str,
        switch: str,
    ):
        """
        :param coin: str
        :param switch: str "ON" or "OFF"
        """
        payload = {
            "coin": coin,
            "collateralSwitch": switch,
        }

        res = await self._request(
            method="POST",
            path=Account.SET_COLLATERAL_COIN,
            query=payload,
        )
        return res

    async def get_fee_rates(
        self,
        product_symbol: str = None,
        category: str = None,
    ):
        """
        Get the trading fee rate
        if product_symbol is not specified, pls specify the category

        :param product_symbol: str
        :param category: str Product type. spot, linear, inverse, option
        """
        payload = {}
        if product_symbol is not None:
            payload["category"] = self.ptm.get_exchange_type(Common.BYBIT, product_symbol)
            payload["symbol"] = self.ptm.get_exchange_symbol(Common.BYBIT, product_symbol)

        if category is not None:
            payload["category"] = category

        res = await self._request(
            method="GET",
            path=Account.GET_FEE_RATE,
            query=payload,
        )
        return res

    async def get_account_info(self):
        res = await self._request(
            method="GET",
            path=Account.GET_ACCOUNT_INFO,
            query=None,
        )
        return res

    async def get_transaction_log(
        self,
        category: str = None,
        coin: str = None,
        startTime: int = None,
        limit: int = 20,
    ):
        """
        :param category: str
        :param coin: str
        :param startTime: int
        :param limit: int
        """
        payload = {
            "limit": limit,
        }
        if category is not None:
            payload["category"] = category
        if coin is not None:
            payload["currency"] = coin
        if startTime is not None:
            payload["startTime"] = startTime

        res = await self._request(
            method="GET",
            path=Account.GET_TRANSACTION_LOG,
            query=payload,
        )
        return res

    async def set_margin_mode(
        self,
        margin_mode: str,
    ):
        """
        :param margin_mode: str ISOLATED_MARGIN, REGULAR_MARGIN(i.e. Cross margin), PORTFOLIO_MARGIN
        """
        payload = {
            "setMarginMode": margin_mode,
        }

        res = await self._request(
            method="POST",
            path=Account.SET_MARGIN_MODE,
            query=payload,
        )
        return res



from ._http_manager import HTTPManager
from .endpoints.account import DeliveryAccount, FutureAccount, SpotAccount


class AccountHTTP(HTTPManager):
    def get_futures_account(
        self,
        ccy: str = "usdt",  # or "btc"
    ):
        """
        :param ccy: str
        """
        path_params = {
            "settle": ccy,
        }

        res = self._request(
            method="GET",
            path=FutureAccount.QUERY_FUTURES_ACCOUNT,
            path_params=path_params,
        )
        return res

    def get_futures_account_book(
        self,
        ccy: str = "usdt",  # or "btc"
        contract: str = None,
        limit: int = None,
        offset: int = None,
        from_time: int = None,
        to_time: int = None,
        change_type: str = None,
    ):
        """
        :param ccy: str
        :param contract: str
        :param limit: int
        :param offset: int
        :param from_time: int
        :param to_time: int
        :param change_type: str
        """
        path_params = {
            "settle": ccy,
        }

        payload = {}
        if contract:
            payload["contract"] = contract
        if limit:
            payload["limit"] = limit
        if offset:
            payload["offset"] = offset
        if from_time:
            payload["from"] = from_time
        if to_time:
            payload["to"] = to_time
        if change_type:
            payload["type"] = change_type

        res = self._request(
            method="GET",
            path=FutureAccount.QUERY_ACCOUNT_BOOK,
            path_params=path_params,
            query=payload,
        )
        return res

    def get_delivery_account(
        self,
        ccy: str = "usdt",
    ):
        """
        :param ccy: str
        """
        path_params = {
            "settle": ccy,
        }

        res = self._request(
            method="GET",
            path=DeliveryAccount.QUERY_DELIVERY_ACCOUNT,
            path_params=path_params,
        )
        return res

    def get_delivery_account_book(
        self,
        ccy: str = "usdt",
        limit: int = None,
        offset: int = None,
        from_time: int = None,
        to_time: int = None,
        change_type: str = None,
    ):
        """
        :param ccy: str
        :param limit: int
        :param offset: int
        :param from_time: int
        :param to_time: int
        :param change_type: str
        """
        path_params = {
            "settle": ccy,
        }

        payload = {}
        if limit:
            payload["limit"] = limit
        if offset:
            payload["offset"] = offset
        if from_time:
            payload["from"] = from_time
        if to_time:
            payload["to"] = to_time
        if change_type:
            payload["type"] = change_type

        res = self._request(
            method="GET",
            path=DeliveryAccount.QUERY_ACCOUNT_BOOK,
            path_params=path_params,
            query=payload,
        )
        return res

    def get_spot_account(
        self,
        ccy: str = None,
    ):
        """
        :param ccy: str
        """
        payload = {}
        if ccy:
            payload["currency"] = ccy

        res = self._request(
            method="GET",
            path=SpotAccount.QUERY_SPOT_ACCOUNT,
            query=payload,
        )
        return res

    def get_spot_account_book(
        self,
        ccy: str = None,
        from_timestamp: int = None,
        to_timestamp: int = None,
        page: int = None,
        limit: int = None,
        type_: str = None,
        code: str = None,
    ):
        """
        :param ccy: str
        :param from_ts: int
        :param to_ts: int
        :param page: int
        :param limit: int
        :param type_: str
        :param code: str
        """
        payload = {}
        if ccy:
            payload["currency"] = ccy
        if from_timestamp:
            payload["from"] = from_timestamp
        if to_timestamp:
            payload["to"] = to_timestamp
        if page:
            payload["page"] = page
        if limit:
            payload["limit"] = limit
        if code:
            payload["code"] = code
        elif type_:
            payload["type"] = type_

        res = self._request(
            method="GET",
            path=SpotAccount.QUERY_ACCOUNT_BOOK,
            query=payload,
        )
        return res



from ._trade_http import TradeHTTP
from ._account_http import AccountHTTP
from ._market_http import MarketHTTP


class Client(
    TradeHTTP,
    AccountHTTP,
    MarketHTTP,
):
    def __init__(self, **args):
        super().__init__(**args)

    async def __aenter__(self):
        await self.async_init()
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.close()

    async def close(self):
        """Close the client and clean up resources."""
        if hasattr(self, 'session') and self.session is not None:
            await self.session.aclose()
            self.session = None



from ._http_manager import HTTPManager
from .endpoints.market import DeliveryMarket, FutureMarket, SpotMarket
from ..utils.common import Common


class MarketHTTP(HTTPManager):
    def get_all_futures_contracts(
        self,
        ccy: str = "usdt",  # or "btc"
        limit: int = None,
        offset: int = None,
    ):
        """
        :param ccy: str
        :param limit: int
        :param offset: int
        """
        path_params = {
            "settle": ccy,
        }

        payload = {}
        if limit:
            payload["limit"] = limit
        if offset:
            payload["offset"] = offset

        res = self._request(
            method="GET",
            path=FutureMarket.GET_ALL_CONTRACTS,
            path_params=path_params,
            query=payload,
            signed=False,
        )
        return res

    def get_a_single_futures_contract(
        self,
        product_symbol: str,
        ccy: str = "usdt",  # or "btc"
    ):
        """
        :param product_symbol: str
        :param ccy: str
        """
        path_params = {
            "settle": ccy,
            "contract": self.ptm.get_exchange_symbol(Common.GATEIO, product_symbol),
        }

        res = self._request(
            method="GET",
            path=FutureMarket.GET_A_SINGLE_CONTRACT,
            path_params=path_params,
            signed=False,
        )
        return res

    def get_contract_order_book(
        self,
        product_symbol: str,
        ccy: str = "usdt",  # or "btc"
        path: str = "futures",
        interval: str = None,
        limit: int = None,
        with_id: bool = False,
    ):
        """
        :param product_symbol: int
        :param ccy: str
        :param path: str
        :param interval: int
        :param limit: int
        :param with_id: bool
        """
        path_params = {
            "settle": ccy,
        }

        payload = {
            "contract": self.ptm.get_exchange_symbol(Common.GATEIO, product_symbol),
        }
        if interval:
            payload["interval"] = interval
        if limit:
            payload["limit"] = limit
        if with_id:
            payload["with_id"] = with_id

        if path == "futures":
            path_ = FutureMarket.ORDER_BOOK
        elif path == "delivery":
            path_ = DeliveryMarket.ORDER_BOOK
        else:
            raise ValueError(f"Unsupported path: {path}")

        res = self._request(
            method="GET",
            path=path_,
            path_params=path_params,
            query=payload,
            signed=False,
        )
        return res

    def get_contract_kline(
        self,
        product_symbol: str,
        ccy: str = "usdt",  # or "btc"
        path: str = "futures",
        from_timestamp: int = None,
        to_timestamp: int = None,
        limit: int = None,
        interval: str = None,
    ):
        """
        :param product_symbol: int
        :param ccy: str
        :param path: str
        :param from_timestamp : int
        :param to_timestamp : int
        :param limit: bool
        :param interval: str
        """
        path_params = {
            "settle": ccy,
        }

        payload = {
            "contract": self.ptm.get_exchange_symbol(Common.GATEIO, product_symbol),
        }
        if from_timestamp:
            payload["from"] = from_timestamp
        if to_timestamp:
            payload["to"] = to_timestamp
        if limit:
            payload["limit"] = limit
        if interval:
            payload["interval"] = interval

        if path == "futures":
            path_ = FutureMarket.GET_KLINE
        elif path == "delivery":
            path_ = DeliveryMarket.GET_KLINE
        else:
            raise ValueError(f"Unsupported path: {path}")

        res = self._request(
            method="GET",
            path=path_,
            path_params=path_params,
            query=payload,
            signed=False,
        )
        return res

    def get_contract_list_tickers(
        self,
        product_symbol: str,
        ccy: str = "usdt",  # or "btc"
        path: str = "futures",
    ):
        """
        :param product_symbol: int
        :param ccy: str
        :param path: str
        """
        path_params = {
            "settle": ccy,
        }

        payload = {
            "contract": self.ptm.get_exchange_symbol(Common.GATEIO, product_symbol),
        }

        if path == "futures":
            path_ = FutureMarket.LIST_TICKERS
        elif path == "delivery":
            path_ = DeliveryMarket.LIST_TICKERS
        else:
            raise ValueError(f"Unsupported path: {path}")

        res = self._request(
            method="GET",
            path=path_,
            path_params=path_params,
            query=payload,
            signed=False,
        )
        return res

    def get_futures_funding_rate_history(
        self,
        product_symbol: str,
        ccy: str = "usdt",  # or "btc"
        limit: int = None,
        from_timestamp: int = None,
        to_timestamp: int = None,
    ):
        """
        :param product_symbol: int
        :param ccy: str
        :param limit: int
        :param from_timestamp: int
        :param to_timestamp: int
        """
        path_params = {
            "settle": ccy,
        }

        payload = {
            "contract": self.ptm.get_exchange_symbol(Common.GATEIO, product_symbol),
        }
        if limit:
            payload["limit"] = limit
        if from_timestamp:
            payload["from"] = from_timestamp
        if to_timestamp:
            payload["to"] = to_timestamp

        res = self._request(
            method="GET",
            path=FutureMarket.FUNDING_RATE_HISTORY,
            path_params=path_params,
            query=payload,
            signed=False,
        )
        return res

    def get_all_delivery_contracts(self):
        path_params = {
            "settle": "usdt",
        }

        res = self._request(
            method="GET",
            path=DeliveryMarket.GET_ALL_CONTRACTS,
            path_params=path_params,
            signed=False,
        )
        return res

    def get_spot_all_currency_pairs(self):
        res = self._request(
            method="GET",
            path=SpotMarket.GET_ALL_CURRENCY_PAIRS,
            signed=False,
        )
        return res

    def get_spot_order_book(
        self,
        product_symbol: str,
        interval: str = None,
        limit: int = None,
        with_id: bool = False,
    ):
        """
        :param product_symbol: int
        :param interval: int
        :param limit: int
        :param with_id: bool
        """
        payload = {
            "currency_pair": self.ptm.get_exchange_symbol(Common.GATEIO, product_symbol),
        }
        if interval:
            payload["interval"] = interval
        if limit:
            payload["limit"] = limit
        if with_id:
            payload["with_id"] = with_id

        res = self._request(
            method="GET",
            path=SpotMarket.ORDER_BOOK,
            query=payload,
            signed=False,
        )
        return res

    def get_spot_kline(
        self,
        product_symbol: str,
        from_timestamp: int = None,
        to_timestamp: int = None,
        limit: int = None,
        interval: str = None,
    ):
        """
        :param product_symbol: int
        :param from_timestamp : int
        :param to_timestamp : int
        :param limit: bool
        :param interval: str
        """
        payload = {
            "currency_pair": self.ptm.get_exchange_symbol(Common.GATEIO, product_symbol),
        }
        if from_timestamp:
            payload["from"] = from_timestamp
        if to_timestamp:
            payload["to"] = to_timestamp
        if limit:
            payload["limit"] = limit
        if interval:
            payload["interval"] = interval

        res = self._request(
            method="GET",
            path=SpotMarket.GET_KLINE,
            query=payload,
            signed=False,
        )
        return res

    def get_spot_list_tickers(
        self,
        product_symbol: str,
        timezone: str = None,
    ):
        """
        :param product_symbol: int
        :param timezone: str
        """

        payload = {
            "currency_pair": self.ptm.get_exchange_symbol(Common.GATEIO, product_symbol),
        }
        if timezone:
            payload["timezone"] = timezone

        res = self._request(
            method="GET",
            path=SpotMarket.LIST_TICKERS,
            query=payload,
            signed=False,
        )
        return res



from enum import Enum


class Asset(str, Enum):
    USERVAULTEQUITIES = "userVaultEquities"

    def __str__(self) -> str:
        return self.value

    def __repr__(self) -> str:
        return repr(self.value)



from ._http_manager import HTTPManager
from .endpoints.trade import Trade
from ..utils.common import Common


class TradeHTTP(HTTPManager):
    def place_order(
        self,
        product_symbol: str,
        tdMode: str,
        side: str,
        ordType: str,
        sz: str,
        ccy: str = None,
        clOrdId: str = None,
        tag: str = None,
        posSide: str = None,
        px: str = None,
        pxUsd: str = None,
        pxVol: str = None,
        reduceOnly: bool = False,
        tgtCcy: str = None,
        banAmend: bool = False,
        quickMgnType: str = None,
        stpId: str = None,
        stpMode: str = None,
    ):
        """
        :param product_symbol: str
        :param tdMode: str
        :param side: str
        :param ordType: str
        :param sz: str
        :param ccy: str
        :param clOrdId: str
        :param tag: str
        :param posSide: str
        :param px: str
        :param pxUsd: str
        :param pxVol: str
        :param reduceOnly: bool
        :param tgtCcy: str
        :param banAmend: bool
        :param quickMgnType: str
        :param stpId: str
        :param stpMode: str
        """
        payload = {
            "instId": self.ptm.get_exchange_symbol(Common.OKX, product_symbol),
            "tdMode": tdMode,
            "side": side,
            "ordType": ordType,
            "sz": sz,
        }
        if ccy is not None:
            payload["ccy"] = ccy
        if clOrdId is not None:
            payload["clOrdId"] = clOrdId
        if tag is not None:
            payload["tag"] = tag
        if posSide is not None:
            payload["posSide"] = posSide
        if px is not None:
            payload["px"] = px
        if pxUsd is not None:
            payload["pxUsd"] = pxUsd
        if pxVol is not None:
            payload["pxVol"] = pxVol
        if reduceOnly is not None:
            payload["reduceOnly"] = reduceOnly
        if tgtCcy is not None:
            payload["tgtCcy"] = tgtCcy
        if banAmend is not None:
            payload["banAmend"] = banAmend
        if quickMgnType is not None:
            payload["quickMgnType"] = quickMgnType
        if stpId is not None:
            payload["stpId"] = stpId
        if stpMode is not None:
            payload["stpMode"] = stpMode

        return self._request(
            method="POST",
            path=Trade.PLACE_ORDER,
            query=payload,
        )

    def place_market_order(
        self,
        product_symbol: str,
        tdMode: str,
        side: str,
        sz: str,
        reduceOnly: bool = None,
        ccy: str = None,
    ):
        return self.place_order(
            product_symbol=product_symbol,
            tdMode=tdMode,
            side=side,
            ordType="market",
            sz=sz,
            reduceOnly=reduceOnly,
            ccy=ccy,
        )

    def place_market_buy_order(
        self,
        product_symbol: str,
        tdMode: str,  # cash or cross
        sz: str,
        reduceOnly: bool = None,
        ccy: str = None,
    ):
        return self.place_market_order(
            product_symbol=product_symbol,
            tdMode=tdMode,
            side="buy",
            sz=sz,
            reduceOnly=reduceOnly,
            ccy=ccy,
        )

    def place_market_sell_order(
        self,
        product_symbol: str,
        tdMode: str,
        sz: str,
        reduceOnly: bool = None,
        ccy: str = None,
    ):
        return self.place_market_order(
            product_symbol=product_symbol,
            tdMode=tdMode,
            side="sell",
            sz=sz,
            reduceOnly=reduceOnly,
            ccy=ccy,
        )

    def place_limit_order(
        self,
        product_symbol: str,
        tdMode: str,
        side: str,
        sz: str,
        px: str,
        reduceOnly: bool = None,
        ccy: str = None,
    ):
        return self.place_order(
            product_symbol=product_symbol,
            tdMode=tdMode,
            side=side,
            ordType="limit",
            sz=sz,
            px=px,
            reduceOnly=reduceOnly,
            ccy=ccy,
        )

    def place_limit_buy_order(
        self,
        product_symbol: str,
        tdMode: str,
        sz: str,
        px: str,
        reduceOnly: bool = None,
        ccy: str = None,
    ):
        return self.place_limit_order(
            product_symbol=product_symbol,
            tdMode=tdMode,
            side="buy",
            sz=sz,
            px=px,
            reduceOnly=reduceOnly,
            ccy=ccy,
        )

    def place_limit_sell_order(
        self,
        product_symbol: str,
        tdMode: str,
        sz: str,
        px: str,
        reduceOnly: bool = None,
        ccy: str = None,
    ):
        return self.place_limit_order(
            product_symbol=product_symbol,
            tdMode=tdMode,
            side="sell",
            sz=sz,
            px=px,
            reduceOnly=reduceOnly,
            ccy=ccy,
        )

    def place_post_only_limit_order(
        self,
        product_symbol: str,
        tdMode: str,
        side: str,
        sz: str,
        px: str,
        reduceOnly: bool = None,
        ccy: str = None,
    ):
        return self.place_order(
            product_symbol=product_symbol,
            tdMode=tdMode,
            side=side,
            ordType="post_only",
            sz=sz,
            px=px,
            reduceOnly=reduceOnly,
            ccy=ccy,
        )

    def place_post_only_limit_buy_order(
        self,
        product_symbol: str,
        tdMode: str,
        sz: str,
        px: str,
        reduceOnly: bool = None,
        ccy: str = None,
    ):
        return self.place_post_only_limit_order(
            product_symbol=product_symbol,
            tdMode=tdMode,
            side="buy",
            sz=sz,
            px=px,
            reduceOnly=reduceOnly,
            ccy=ccy,
        )

    def place_post_only_limit_sell_order(
        self,
        product_symbol: str,
        tdMode: str,
        sz: str,
        px: str,
        reduceOnly: bool = None,
        ccy: str = None,
    ):
        return self.place_post_only_limit_order(
            product_symbol=product_symbol,
            tdMode=tdMode,
            side="sell",
            sz=sz,
            px=px,
            reduceOnly=reduceOnly,
            ccy=ccy,
        )

    def cancel_order(
        self,
        product_symbol: str,
        ordId: str = None,
        clOrdId: str = None,
    ):
        """
        :param product_symbol: str
        :param ordId: str
        :param clOrdId: str
        """
        payload = {
            "instId": self.ptm.get_exchange_symbol(Common.OKX, product_symbol),
        }
        if ordId is not None:
            payload["ordId"] = ordId
        if clOrdId is not None:
            payload["clOrdId"] = clOrdId

        return self._request(
            method="POST",
            path=Trade.CANCEL_ORDER,
            query=payload,
        )

    def cancel_all_orders(
        self,
        product_symbol: str = None,
    ):
        """
        :param product_symbol: str
        :param ordId: str
        :param clOrdId: str
        """
        payload = []

        all_orders = self.get_order_list()
        all_orders = all_orders["data"]
        if product_symbol is not None:
            exchange_symbol = self.ptm.get_exchange_symbol(product_symbol, Common.OKX)
            for order in all_orders:
                if order["instId"] == exchange_symbol:
                    payload.append(
                        {
                            "instId": order["instId"],
                            "ordId": order["ordId"],
                            "clOrdId": order["clOrdId"],
                        }
                    )
        else:
            for order in all_orders:
                payload.append(
                    {
                        "instId": order["instId"],
                        "ordId": order["ordId"],
                        "clOrdId": order["clOrdId"],
                    }
                )

        return self._request(
            method="POST",
            path=Trade.CANCEL_BATCH_ORDERS,
            query=payload,
        )

    def amend_order(
        self,
        product_symbol: str,
        ordId: str = None,
        clOrdId: str = None,
        newSz: str = None,
        newPx: str = None,
        newPxUsd: str = None,
        newPxVol: str = None,
        cxlOnFail: str = None,
        reqId: str = None,
    ):
        """
        :param product_symbol: str
        :param ordId: str
        :param clOrdId: str
        :param newSz: str
        :param newPx: str
        :param newPxUsd: str
        :param newPxVol: str
        :param cxlOnFail: str
        :param reqId: str
        """
        payload = {
            "instId": self.ptm.get_exchange_symbol(Common.OKX, product_symbol),
        }
        if ordId is not None:
            payload["ordId"] = ordId
        if clOrdId is not None:
            payload["clOrdId"] = clOrdId
        if newSz is not None:
            payload["newSz"] = newSz
        if newPx is not None:
            payload["newPx"] = newPx
        if newPxUsd is not None:
            payload["newPxUsd"] = newPxUsd
        if newPxVol is not None:
            payload["newPxVol"] = newPxVol
        if cxlOnFail is not None:
            payload["cxlOnFail"] = cxlOnFail
        if reqId is not None:
            payload["reqId"] = reqId

        return self._request(
            method="POST",
            path=Trade.AMEND_ORDER,
            query=payload,
        )

    def amend_multiple_orders(
        self,
        product_symbol: str,
        ordId: str = None,
        clOrdId: str = None,
        newSz: str = None,
        newPx: str = None,
        newPxUsd: str = None,
        newPxVol: str = None,
        cxlOnFail: str = None,
        reqId: str = None,
    ):
        """
        :param product_symbol: str
        :param ordId: str
        :param clOrdId: str
        :param newSz: str
        :param newPx: str
        :param newPxUsd: str
        :param newPxVol: str
        :param cxlOnFail: str
        :param reqId: str
        """
        payload = {
            "instId": self.ptm.get_exchange_symbol(Common.OKX, product_symbol),
        }
        if ordId is not None:
            payload["ordId"] = ordId
        if clOrdId is not None:
            payload["clOrdId"] = clOrdId
        if newSz is not None:
            payload["newSz"] = newSz
        if newPx is not None:
            payload["newPx"] = newPx
        if newPxUsd is not None:
            payload["newPxUsd"] = newPxUsd
        if newPxVol is not None:
            payload["newPxVol"] = newPxVol
        if cxlOnFail is not None:
            payload["cxlOnFail"] = cxlOnFail
        if reqId is not None:
            payload["reqId"] = reqId

        return self._request(
            method="POST",
            path=Trade.AMEND_BATCH_ORDER,
            query=payload,
        )

    def close_positions(
        self,
        product_symbol: str,
        mgnMode: str,
        posSide: str = None,
        ccy: str = None,
    ):
        """
        :param product_symbol: str
        :param mgnMode: str
        :param posSide: str
        :param ccy: str
        """
        payload = {
            "instId": self.ptm.get_exchange_symbol(product_symbol, Common.OKX),
            "mgnMode": mgnMode,
        }
        if posSide is not None:
            payload["posSide"] = posSide
        if ccy is not None:
            payload["ccy"] = ccy

        return self._request(
            method="POST",
            path=Trade.CLOSE_POSITION,
            query=payload,
        )

    def get_order(
        self,
        product_symbol: str,
        ordId: str = None,
        clOrdId: str = None,
    ):
        """
        :param product_symbol: str
        :param ordId: str
        :param clOrdId: str
        """
        payload = {
            "instId": self.ptm.get_exchange_symbol(Common.OKX, product_symbol),
        }
        if ordId is not None:
            payload["ordId"] = ordId
        if clOrdId is not None:
            payload["clOrdId"] = clOrdId

        res = self._request(
            method="GET",
            path=Trade.ORDER_INFO,
            query=payload,
        )
        return res

    def get_order_list(
        self,
        instType: str = None,
        uly: str = None,
        instFamily: str = None,
        product_symbol: str = None,
        ordType: str = None,
        state: str = None,
        limit: str = None,
    ):
        """
        :param instType: str (SPOT, MARGIN, SWAP, FUTURES, OPTION)
        :param uly: str
        :param instFamily: str
        :param product_symbol: str
        :param ordType: str
        :param state: str
        :param limit: str
        """
        payload = {}
        if instType is not None:
            payload["instType"] = instType
        if uly is not None:
            payload["uly"] = uly
        if instFamily is not None:
            payload["instFamily"] = instFamily
        if product_symbol is not None:
            payload["instId"] = self.ptm.get_exchange_symbol(Common.OKX, product_symbol)
        if ordType is not None:
            payload["ordType"] = ordType
        if state is not None:
            payload["state"] = state
        if limit is not None:
            payload["limit"] = limit

        res = self._request(
            method="GET",
            path=Trade.ORDERS_PENDING,
            query=payload,
        )
        return res

    def get_orders_history(
        self,
        instType: str,
        uly: str = None,
        instFamily: str = None,
        product_symbol: str = None,
        ordType: str = None,
        state: str = None,
        category: str = None,
        begin: str = None,
        end: str = None,
        limit: str = None,
    ):
        """
        :param instType: str (SPOT, MARGIN, SWAP, FUTURES, OPTION)
        :param uly: str
        :param instFamily: str
        :param product_symbol: str
        :param ordType: str
        :param state: str
        :param category: str
        :param begin: str
        :param end: str
        :param limit: str
        """
        payload = {
            "instType": instType,
        }
        if uly is not None:
            payload["uly"] = uly
        if instFamily is not None:
            payload["instFamily"] = instFamily
        if product_symbol is not None:
            payload["instId"] = self.ptm.get_exchange_symbol(Common.OKX, product_symbol)
        if ordType is not None:
            payload["ordType"] = ordType
        if state is not None:
            payload["state"] = state
        if category is not None:
            payload["category"] = category
        if begin is not None:
            payload["begin"] = begin
        if end is not None:
            payload["end"] = end
        if limit is not None:
            payload["limit"] = limit

        res = self._request(
            method="GET",
            path=Trade.ORDERS_HISTORY,
            query=payload,
        )
        return res

    def get_orders_history_archive(
        self,
        instType: str,
        uly: str = None,
        instFamily: str = None,
        product_symbol: str = None,
        ordType: str = None,
        state: str = None,
        category: str = None,
        begin: str = None,
        end: str = None,
        limit: str = None,
    ):
        """
        :param instType: str (SPOT, MARGIN, SWAP, FUTURES, OPTION)
        :param uly: str
        :param instFamily: str
        :param product_symbol: str
        :param ordType: str
        :param state: str
        :param category: str
        :param begin: str
        :param end: str
        :param limit: str
        """
        payload = {
            "instType": instType,
        }
        if uly is not None:
            payload["uly"] = uly
        if instFamily is not None:
            payload["instFamily"] = instFamily
        if product_symbol is not None:
            payload["instId"] = self.ptm.get_exchange_symbol(Common.OKX, product_symbol)
        if ordType is not None:
            payload["ordType"] = ordType
        if state is not None:
            payload["state"] = state
        if category is not None:
            payload["category"] = category
        if begin is not None:
            payload["begin"] = begin
        if end is not None:
            payload["end"] = end
        if limit is not None:
            payload["limit"] = limit

        res = self._request(
            method="GET",
            path=Trade.ORDERS_HISTORY_ARCHIVE,
            query=payload,
        )
        return res

    def get_fills(
        self,
        instType: str = None,
        uly: str = None,
        instFamily: str = None,
        product_symbol: str = None,
        ordId: str = None,
        subType: str = None,
        begin: str = None,
        end: str = None,
        limit: str = None,
    ):
        """
        :param instType: str (SPOT, MARGIN, SWAP, FUTURES, OPTION)
        :param uly: str
        :param instFamily: str
        :param product_symbol: str
        :param ordId: str
        :param subType: str
        :param begin: str
        :param end: str
        :param limit: str
        """
        payload = {}
        if instType is not None:
            payload["instType"] = instType
        if uly is not None:
            payload["uly"] = uly
        if instFamily is not None:
            payload["instFamily"] = instFamily
        if product_symbol is not None:
            payload["instId"] = self.ptm.get_exchange_symbol(Common.OKX, product_symbol)
        if ordId is not None:
            payload["ordId"] = ordId
        if subType is not None:
            payload["subType"] = subType
        if begin is not None:
            payload["begin"] = begin
        if end is not None:
            payload["end"] = end
        if limit is not None:
            payload["limit"] = limit

        res = self._request(
            method="GET",
            path=Trade.ORDER_FILLS,
            query=payload,
        )
        return res

    def get_fills_history(
        self,
        instType: str,
        uly: str = None,
        instFamily: str = None,
        product_symbol: str = None,
        ordId: str = None,
        subType: str = None,
        begin: str = None,
        end: str = None,
        limit: str = None,
    ):
        """
        :param instType: str (SPOT, MARGIN, SWAP, FUTURES, OPTION)
        :param uly: str
        :param instFamily: str
        :param product_symbol: str
        :param ordId: str
        :param subType: str
        :param begin: str
        :param end: str
        :param limit: str
        """
        payload = {
            "instType": instType,
        }
        if uly is not None:
            payload["uly"] = uly
        if instFamily is not None:
            payload["instFamily"] = instFamily
        if product_symbol is not None:
            payload["instId"] = self.ptm.get_exchange_symbol(Common.OKX, product_symbol)
        if ordId is not None:
            payload["ordId"] = ordId
        if subType is not None:
            payload["subType"] = subType
        if begin is not None:
            payload["begin"] = begin
        if end is not None:
            payload["end"] = end
        if limit is not None:
            payload["limit"] = limit

        res = self._request(
            method="GET",
            path=Trade.ORDERS_FILLS_HISTORY,
            query=payload,
        )
        return res

    def get_account_rate_limit(self):
        res = self._request(
            method="GET",
            path=Trade.ACCOUNT_RATE_LIMIT,
            query=None,
        )
        return res



import re
import polars as pl
from typing import Dict
from dataclasses import dataclass, asdict
from ...utils.decimal_utils import reverse_decimal_places
from ...utils.common import Common
from ...utils.common_dataframe import to_dataframe


@dataclass
class MarketInfo:
    exchange: str
    exchange_symbol: str
    product_symbol: str
    product_type: str
    exchange_type: str
    price_precision: str
    size_precision: str
    min_size: str
    base_currency: str = ""
    quote_currency: str = ""
    min_notional: str = "0"
    multiplier: str = "1"

    # contract
    size_per_contract: str = "1"

    def to_dict(self) -> Dict[str, str]:
        return asdict(self)


def strip_number(s: str) -> str:
    return re.sub(r"^\d+", "", s)


def clean_symbol(symbol: str) -> str:
    return re.sub(r"[$_]", "", symbol)


def format_product_symbol(symbol: str) -> str:
    """
    - BTCUSDT → BTC-USDT-SWAP
    - BTCUSDT-04APR25 → BTC-USDT-04APR25-FUTURES
    - ETH-25APR25 → ETH-25APR25-FUTURES
    - AAVEUSD → AAVE-USD
    - ETHUSDH25 → ETH-USD-H25
    """
    match = re.match(r"([A-Z]+)(USD[T]?)\-([0-9]{2}[A-Z]{3}[0-9]{2})$", symbol)
    if match:
        base, quote, date = match.groups()
        return f"{base}-{quote}-{date}-FUTURES"

    # ETH-25APR25 --> ETH-25APR25-FUTURES
    match = re.match(r"([A-Z]+)-(\d+[A-Z]{3}\d{2})$", symbol)
    if match:
        base, date = match.groups()
        return f"{base}-{date}-FUTURES"

    # ETHUSDH25 --> ETH-USD-H25-FUTURES
    match = re.match(r"([A-Z]+)(USD[T]?)([HMUZ]\d{2})$", symbol)
    if match:
        base, quote, date = match.groups()
        return f"{base}-{quote}-{date}-FUTURES"

    # AAVEUSD --> AAVE-USD-SWAP
    match = re.match(r"([A-Z]+)(USD[T]?)$", symbol)
    if match:
        base, quote = match.groups()
        return f"{base}-{quote}-SWAP"

    quote_currencies = {"USDT", "USDC", "PERP", "USD"}

    matched_quote = next((quote for quote in quote_currencies if symbol.endswith(quote)), None)

    if matched_quote:
        base = symbol[: -len(matched_quote)]
        return f"{base}-{matched_quote}-SWAP"

    return f"{symbol}-SWAP"


async def bybit() -> pl.DataFrame:
    from ..bybit._market_http import MarketHTTP

    market_http = MarketHTTP()
    await market_http.async_init()

    markets = []

    res_linear = await market_http.get_instruments_info(category="linear")
    df_linear = to_dataframe(res_linear["result"]["list"]) if "list" in res_linear.get("result", {}) else pl.DataFrame()
    for market in df_linear.iter_rows(named=True):
        if market["contractType"] == "LinearFutures":
            product_type = "futures"
        else:
            product_type = "swap"
        markets.append(
            MarketInfo(
                exchange=Common.BYBIT,
                exchange_symbol=market["symbol"],
                product_symbol=format_product_symbol(strip_number(market["symbol"])),
                product_type=product_type,
                exchange_type="linear",
                base_currency=strip_number(market["baseCoin"]),
                quote_currency=market["quoteCoin"],
                price_precision=market["priceFilter"]["tickSize"],
                size_precision=market["lotSizeFilter"]["qtyStep"],
                min_size=market["lotSizeFilter"]["minOrderQty"],
                min_notional=market["lotSizeFilter"].get("minNotionalValue", "0"),
            )
        )

    res_inverse = await market_http.get_instruments_info(category="inverse")
    df_inverse = (
        to_dataframe(res_inverse["result"]["list"]) if "list" in res_inverse.get("result", {}) else pl.DataFrame()
    )
    for market in df_inverse.iter_rows(named=True):
        if market["contractType"] == "InverseFutures":
            product_type = "futures"
        else:
            product_type = "swap"
        markets.append(
            MarketInfo(
                exchange=Common.BYBIT,
                exchange_symbol=market["symbol"],
                product_symbol=format_product_symbol(market["symbol"]),
                product_type=product_type,
                exchange_type="inverse",
                base_currency=strip_number(market["baseCoin"]),
                quote_currency=market["quoteCoin"],
                price_precision=market["priceFilter"]["tickSize"],
                size_precision=market["lotSizeFilter"]["qtyStep"],
                min_size=market["lotSizeFilter"]["minOrderQty"],
            )
        )

    res_spot = await market_http.get_instruments_info(category="spot")
    df_spot = to_dataframe(res_spot["result"]["list"]) if "list" in res_spot.get("result", {}) else pl.DataFrame()
    for market in df_spot.iter_rows(named=True):
        if not market["symbol"].endswith(("USDT", "USDC")):
            continue
        markets.append(
            MarketInfo(
                exchange=Common.BYBIT,
                exchange_symbol=market["symbol"],
                product_symbol=f"{market['symbol'][:-4]}-{market['symbol'][-4:]}-SPOT",
                product_type="spot",
                exchange_type="spot",
                base_currency=strip_number(market["baseCoin"]),
                quote_currency=market["quoteCoin"],
                price_precision=market["priceFilter"]["tickSize"],
                size_precision=market["lotSizeFilter"]["basePrecision"],
                min_size=market["lotSizeFilter"]["minOrderQty"],
                min_notional=market["lotSizeFilter"].get("minNotionalValue", "0"),
            )
        )
    markets = [market.to_dict() for market in markets]
    return pl.DataFrame(markets)


async def okx() -> pl.DataFrame:
    from ..okx._public_http import PublicHTTP

    public_http = PublicHTTP()
    await public_http.async_init()

    markets = []

    res_swap = await public_http.get_public_instruments(instType="SWAP")
    df_swap = to_dataframe(res_swap["data"]) if "data" in res_swap else pl.DataFrame()
    for market in df_swap.iter_rows(named=True):
        base = strip_number(market["baseCcy"])
        quote = market["quoteCcy"]

        if not base or not quote:
            parts = market["instId"].split("-")
            if len(parts) >= 2:
                base, quote = parts[0], parts[1]

        markets.append(
            MarketInfo(
                exchange=Common.OKX,
                exchange_symbol=market["instId"],
                product_symbol=strip_number(market["instId"]),
                product_type="swap",
                exchange_type=market["instType"],
                base_currency=base,
                quote_currency=quote,
                price_precision=market["tickSz"],
                size_precision=market["lotSz"],
                min_size=market["minSz"],
                size_per_contract=market["ctVal"],
            )
        )

    res_spot = await public_http.get_public_instruments(instType="SPOT")
    df_spot = to_dataframe(res_spot["data"]) if "data" in res_spot else pl.DataFrame()
    for market in df_spot.iter_rows(named=True):
        if not market["instId"].endswith(("USDT", "USDC", "USD")):
            continue
        markets.append(
            MarketInfo(
                exchange=Common.OKX,
                exchange_symbol=market["instId"],
                product_symbol=market["instId"] + "-SPOT",
                product_type="spot",
                exchange_type=market["instType"],
                base_currency=strip_number(market["baseCcy"]),
                quote_currency=market["quoteCcy"],
                price_precision=market["tickSz"],
                size_precision=market["lotSz"],
                min_size=market["minSz"],
            )
        )

    res_futures = await public_http.get_public_instruments(instType="FUTURES")
    df_futures = to_dataframe(res_futures["data"]) if "data" in res_futures else pl.DataFrame()
    for market in df_futures.iter_rows(named=True):
        base = strip_number(market["baseCcy"])
        quote = market["quoteCcy"]

        if not base or not quote:
            parts = market["instId"].split("-")
            if len(parts) >= 2:
                base, quote = parts[0], parts[1]

        markets.append(
            MarketInfo(
                exchange=Common.OKX,
                exchange_symbol=market["instId"],
                product_symbol=strip_number(market["instId"]),
                product_type="futures",
                exchange_type=market["instType"],
                base_currency=base,
                quote_currency=quote,
                price_precision=market["tickSz"],
                size_precision=market["lotSz"],
                min_size=market["minSz"],
            )
        )

    markets = [market.to_dict() for market in markets]
    return pl.DataFrame(markets)


async def bitmart() -> pl.DataFrame:
    from ..bitmart._market_http import MarketHTTP

    market_http = MarketHTTP()
    await market_http.async_init()

    markets = []
    quote_currencies = {"USDT", "USDC", "USD"}

    res_swap = await market_http.get_contracts_details()
    df_swap = to_dataframe(res_swap.get("data", {}).get("symbols", []))
    for market in df_swap.iter_rows(named=True):
        matched_quote = next(
            (quote for quote in quote_currencies if market["symbol"].endswith(quote)),
            None,
        )

        if matched_quote:
            base = market["symbol"][: -len(matched_quote)]
            product_symbol = f"{base}-{matched_quote}-SWAP"
        else:
            product_symbol = f"{market['symbol']}-SWAP"

        markets.append(
            MarketInfo(
                exchange=Common.BITMART,
                exchange_symbol=market["symbol"],
                product_symbol=product_symbol,
                product_type="swap",
                exchange_type="swap",
                base_currency=strip_number(market["base_currency"]),
                quote_currency=market["quote_currency"],
                price_precision=market["price_precision"],
                size_precision=market["vol_precision"],
                min_size=market["min_volume"],
                size_per_contract=market["contract_size"],
            )
        )

    res_spot = await market_http.get_trading_pairs_details()
    df_spot = to_dataframe(res_spot.get("data", {}).get("symbols", []))
    for market in df_spot.iter_rows(named=True):
        if not market["symbol"].endswith(("USDT", "USDC", "USD")):
            continue

        matched_quote = next(
            (quote for quote in quote_currencies if market["symbol"].endswith(quote)),
            None,
        )

        if matched_quote:
            base = clean_symbol(market["symbol"][: -len(matched_quote)])
            product_symbol = f"{base}-{matched_quote}-SPOT"
        else:
            product_symbol = f"{clean_symbol(market['symbol'])}-SPOT"

        markets.append(
            MarketInfo(
                exchange=Common.BITMART,
                exchange_symbol=market["symbol"],
                product_symbol=product_symbol,
                product_type="spot",
                exchange_type="spot",
                base_currency=strip_number(market["base_currency"]),
                quote_currency=market["quote_currency"],
                price_precision=reverse_decimal_places(market["price_max_precision"]),
                size_precision=market["quote_increment"],
                min_size=market["base_min_size"],
                min_notional=market["min_buy_amount"],
            )
        )

    markets = [market.to_dict() for market in markets]
    return pl.DataFrame(markets)


async def gateio() -> pl.DataFrame:
    from ..gateio._market_http import MarketHTTP

    market_http = MarketHTTP()
    await market_http.async_init()

    markets = []
    quote_currencies = {"USDT", "USDC", "USD"}

    res_futures = await market_http.get_all_futures_contracts()
    df_futures = to_dataframe(res_futures)
    for market in df_futures.iter_rows(named=True):
        matched_quote = next(
            (quote for quote in quote_currencies if market["name"].endswith(quote)),
            None,
        )

        if matched_quote:
            base = clean_symbol(market["name"][: -len(matched_quote)])
            product_symbol = f"{base}-{matched_quote}-SWAP"
        else:
            product_symbol = f"{clean_symbol(market['name'])}-SWAP"

        parts = market["name"].split("_")
        if len(parts) >= 2:
            base, quote = parts[0], parts[1]

        markets.append(
            MarketInfo(
                exchange=Common.GATEIO,
                exchange_symbol=market["name"],
                product_symbol=product_symbol,
                product_type="swap",
                exchange_type="futures",
                base_currency=base,
                quote_currency=quote,
                price_precision=market["order_price_round"],
                size_precision=str(market["order_size_min"]),
                min_size=str(market["order_size_min"]),
                size_per_contract=market["quanto_multiplier"],
            )
        )

    res_delivery = await market_http.get_all_delivery_contracts()
    df_deliver = to_dataframe(res_delivery)
    for market in df_deliver.iter_rows(named=True):
        matched_quote = next(
            (quote for quote in quote_currencies if market["name"].split("_")[1].endswith(quote)),
            None,
        )

        if matched_quote:
            base = clean_symbol(market["name"].split("_")[0])
            product_symbol = f"{base}-{matched_quote}-{market['name'].split('_')[2]}-SWAP"
        else:
            product_symbol = f"{clean_symbol(market['name'])}-SWAP"

        parts = market["name"].split("_")
        if len(parts) >= 2:
            base, quote = parts[0], parts[1]

        markets.append(
            MarketInfo(
                exchange=Common.GATEIO,
                exchange_symbol=market["name"],
                product_symbol=product_symbol,
                product_type="futures",
                exchange_type="delivery",
                base_currency=base,
                quote_currency=quote,
                price_precision=market["order_price_round"],
                size_precision=str(market["order_size_min"]),
                min_size=str(market["order_size_min"]),
                size_per_contract=market["quanto_multiplier"],
            )
        )

    res_spot = await market_http.get_spot_all_currency_pairs()
    df_spot = to_dataframe(res_spot)
    for market in df_spot.iter_rows(named=True):
        if not market["id"].endswith(("USDT", "USDC", "USD")):
            continue
        markets.append(
            MarketInfo(
                exchange=Common.GATEIO,
                exchange_symbol=market["id"],
                product_symbol=f"{market['base']}-{market['quote']}-SPOT",
                product_type="spot",
                exchange_type="spot",
                base_currency=market["base"],
                quote_currency=market["quote"],
                price_precision=reverse_decimal_places(market["precision"]),
                size_precision=reverse_decimal_places(market["amount_precision"]),
                min_size=market["min_base_amount"],
            )
        )

    markets = [market.to_dict() for market in markets]
    return pl.DataFrame(markets)


async def binance() -> pl.DataFrame:
    from ..binance._market_http import MarketHTTP

    market_http = MarketHTTP()
    await market_http.async_init()

    markets = []
    quote_currencies = {"USDT", "USDC", "USD"}

    res_spot = await market_http.get_spot_exchange_info()
    df_spot = to_dataframe(res_spot.get("symbols", []))
    for market in df_spot.iter_rows(named=True):
        if not market["symbol"].endswith(("USDT", "USDC", "USD")):
            continue

        matched_quote = next(
            (quote for quote in quote_currencies if market["symbol"].endswith(quote)),
            None,
        )

        if matched_quote:
            product_symbol = clean_symbol(f"{market['baseAsset']}-{market['quoteAsset']}-SPOT")

        price_filter = next((f for f in market["filters"] if f["filterType"] == "PRICE_FILTER"), {})
        lot_size_filter = next((f for f in market["filters"] if f["filterType"] == "LOT_SIZE"), {})
        min_notional_filter = next((f for f in market["filters"] if f["filterType"] == "MIN_NOTIONAL"), {})

        markets.append(
            MarketInfo(
                exchange=Common.BINANCE,
                exchange_symbol=market["symbol"],
                product_symbol=product_symbol,
                product_type="spot",
                exchange_type="spot",
                base_currency=market["baseAsset"],
                quote_currency=market["quoteAsset"],
                price_precision=price_filter.get("tickSize", "0"),
                size_precision=lot_size_filter.get("stepSize", "0"),
                min_size=lot_size_filter.get("minQty", "0"),
                min_notional=min_notional_filter.get("notional", "0"),
            )
        )

    res_futures = await market_http.get_futures_exchange_info()
    df_futures = to_dataframe(res_futures.get("symbols", []))
    for market in df_futures.iter_rows(named=True):
        if not market["symbol"].endswith(("USDT", "USDC", "USD", "BUSD")):
            continue

        base = strip_number(market["baseAsset"])
        quote = market["quoteAsset"]
        product_symbol = f"{base}-{quote}-SWAP"

        price_filter = next((f for f in market["filters"] if f["filterType"] == "PRICE_FILTER"), {})
        lot_size_filter = next((f for f in market["filters"] if f["filterType"] == "LOT_SIZE"), {})
        min_notional_filter = next((f for f in market["filters"] if f["filterType"] == "MIN_NOTIONAL"), {})

        markets.append(
            MarketInfo(
                exchange=Common.BINANCE,
                exchange_symbol=market["symbol"],
                product_symbol=product_symbol,
                product_type="swap",
                exchange_type=market["contractType"],
                base_currency=base,
                quote_currency=quote,
                price_precision=price_filter.get("tickSize", "0"),
                size_precision=lot_size_filter.get("stepSize", "0"),
                min_size=lot_size_filter.get("minQty", "0"),
                min_notional=min_notional_filter.get("notional", "0"),
            )
        )

    markets = [market.to_dict() for market in markets]
    return pl.DataFrame(markets)


async def hyperliquid() -> pl.DataFrame:
    from ..hyperliquid._market_http import MarketHTTP

    market_http = MarketHTTP()
    await market_http.async_init()

    markets = []

    res_prep = await market_http.meta()
    df_prep = to_dataframe(res_prep.get("universe", []))

    for idx, market in enumerate(df_prep.iter_rows(named=True)):
        coin = market["name"]
        tick = str(reverse_decimal_places(market["szDecimals"]))
        markets.append(
            MarketInfo(
                exchange=Common.HYPERLIQUID,
                exchange_symbol=f'["{coin}", {idx}]',
                product_symbol=f"{coin}-USD-SWAP",
                product_type="swap",
                exchange_type="perpetual",
                base_currency=coin,
                quote_currency="USD",
                price_precision=tick,
                size_precision=tick,
                min_size=tick,
                size_per_contract="1",
            )
        )

    res_spot = await market_http.spot_meta()
    df_tokens = to_dataframe(res_spot.get("tokens", []))
    df_spot = to_dataframe(res_spot.get("universe", []))

    for idx, market in enumerate(df_spot.iter_rows(named=True)):
        # exchange_symbol = market["name"]
        base_i, quote_i = market["tokens"]

        base = df_tokens["name"][base_i]  # e.g. "PURR"
        quote = df_tokens["name"][quote_i]  # e.g. "USDC"
        tick = str(reverse_decimal_places(df_tokens["szDecimals"][base_i]))

        markets.append(
            MarketInfo(
                exchange=Common.HYPERLIQUID,
                exchange_symbol='["{}", {}]'.format(market["name"], 10000 + idx),
                product_symbol=f"{base}-{quote}-SPOT",
                product_type="spot",
                exchange_type="spot",
                base_currency=base,
                quote_currency=quote,
                price_precision=tick,
                size_precision=tick,
                min_size=tick,
            )
        )
    markets = [market.to_dict() for market in markets]
    return pl.DataFrame(markets)



import math


def get_decimal_places(value):
    """Returns the number of decimal places for a given value."""
    if value > 0:
        return int(-math.log10(value))
    return 0  # Avoid errors when value is 0


def reverse_decimal_places(decimal_places):
    """Converts a decimal place count back to its corresponding value."""
    return 10**-decimal_places


# Testing the functions
# print(get_decimal_places(0.01))  # 2
# print(get_decimal_places(0.001))  # 3
# print(get_decimal_places(1))  # -1
# print(get_decimal_places(10))  # -2

# print(reverse_decimal_places(2))  # 0.01
# print(reverse_decimal_places(3))  # 0.001
# print(reverse_decimal_places(-1))  # 1
# print(reverse_decimal_places(-2))  # 10
